/*  Copyright (C) 2004-2011
    ANTONIO JAVIER BARRAGAN, antonio.barragan@diesia.uhu.es
    http://www.uhu.es/antonio.barragan

    Collaborators:
    JOSE MANUEL ANDUJAR, andujar@diesia.uhu.es
	MARIANO J. AZNAR, marianojose.aznar@alu.uhu.es

    DPTO. DE ING. ELECTRONICA, DE SISTEMAS INFORMATICOS Y AUTOMATICA
    ETSI, UNIVERSITY OF HUELVA (SPAIN)

    For more information, please contact with authors.
    
    This software is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _TNT_EXTENSION_HPP_
#define _TNT_EXTENSION_HPP_

/**
 * \file tnt_extension.hpp
 * \brief Implements useful functions for the management of types defined in the TNT library.
 * 
 * \sa For Template Numerical Toolkit (TNT) documentation see http://math.nist.gov/tnt
 */

#include <tnt/tnt.h> // Template Numerical Toolkit (http://math.nist.gov/tnt)

/**
 * \brief Creates an Identity Array2D of the given \c order.
 */
inline TNT::Array2D<double> makeIdentity(int order)
{
	TNT::Array2D<double> I(order,order,0.0);
	for (int i=0;i<order;i++)
		I[i][i]=1.0;
	return I;
}

/**
 * \brief Extracts the \c r row from an Array2D.
 */
template <class T>
inline TNT::Array1D<T> copyrow(const TNT::Array2D<T> &Array, int r)
{
	if (r >= Array.dim1())
	{
		TNT::Array1D<T> A;
		return A;
	}
	int m = Array.dim2();
	TNT::Array1D<T> A(m);
	for (int c=0; c<m; c++)
		A[c] = Array[r][c];
	return A;
}

/**
 * \brief Extracts the \c c column from an Array2D.
 */
template <class T>
inline TNT::Array1D<T> copycolumn(const TNT::Array2D<T> &Array, int c)
{
	if (c >= Array.dim2())
	{
		TNT::Array1D<T> A;
		return A;
	}
	int n = Array.dim1();
	TNT::Array1D<T> A(n);
	for (int r=0; r<n; r++)
		A[r] = Array[r][c];
	return A;
}

/**
 * \brief Extracts the \c m Array2D from an Array3D.
 */
template <class T>
inline TNT::Array2D<T> copyArray2D(const TNT::Array3D<T> &Array, int m)
{
	if (m >= Array.dim1())
	{
		TNT::Array2D<T> A;
		return A;
	}
	int n = Array.dim2();
	int o = Array.dim3();
	TNT::Array2D<T> A(n, o);
	for (int r=0; r<n; r++)
		for (int c=0; c<o; c++)
			A[r][c] = Array[m][r][c];
	return A;
}

/**
 * \brief Injects a row to an Array2D in the position \c r.
 */
template <class T>
inline int injectrow(TNT::Array2D<T> &A2D, const TNT::Array1D<T> &A1D, int r)
{
	int m = A2D.dim2();
	if (m == A1D.dim() && r < A2D.dim1())
	{
		for (int c=0; c<m; c++)
			A2D[r][c] = A1D[c];
		return 0;
	}
	else
		return 1;
}

/**
 * \brief Injects a column to an Array2D in the position \c c.
 */
template <class T>
inline int injectcolumn(TNT::Array2D<T> &A2D, const TNT::Array1D<T> &A1D, int c)
{
	int n = A2D.dim1();
	if (n == A1D.dim() && c < A2D.dim2())
	{
		for (int r=0; r<n; r++)
			A2D[r][c] = A1D[r];
		return 0;
	}
	return 1;
}

/**
 * \brief Injects an Array2D to an Array3D in the position \c m.
 */
template <class T>
inline int injectArray2D(TNT::Array3D<T> &A3D, const TNT::Array2D<T> &A2D, int m)
{
	int n = A3D.dim2();
	int o = A3D.dim3();
	if (n == A2D.dim1() && o == A2D.dim2() && m < A3D.dim1())
	{
		for (int r=0; r<n; r++)
			for (int c=0; c<o; c++)
				A3D[m][r][c] = A2D[r][c];
		return 0;
	}
	else
		return 1;
}

#endif
